@@ -1,45 +1,41 @@
-package Log::Any::Adapter::Test;
-{
-  $Log::Any::Adapter::Test::VERSION = '0.15';
-}
-use Data::Dumper;
-use Log::Any;
-use Test::Builder;
+use 5.008001;
 use strict;
 use warnings;
-use base qw(Log::Any::Adapter::Core);
+
+package Log::Any::Adapter::Test;
+
+our $VERSION = '1.00';
+
+use Log::Any::Adapter::Util qw/dump_one_line/;
+use Test::Builder;
+
+use base qw/Log::Any::Adapter::Base/;
 
 my $tb = Test::Builder->new();
 my @msgs;
 
-sub new {
-    my $class = shift;
-    return bless {@_}, $class;
-}
-
 # All detection methods return true
 #
-foreach my $method ( Log::Any->detection_methods() ) {
-    _make_method( $method, sub { 1 } );
+foreach my $method ( Log::Any::Adapter::Util::detection_methods() ) {
+    no strict 'refs';
+    *{$method} = sub { 1 };
 }
 
 # All logging methods push onto msgs array
 #
-foreach my $method ( Log::Any->logging_methods() ) {
-    _make_method(
-        $method,
-        sub {
-            my ( $self, $msg ) = @_;
-            push(
-                @msgs,
-                {
-                    message  => $msg,
-                    level    => $method,
-                    category => $self->{category}
-                }
-            );
-        }
-    );
+foreach my $method ( Log::Any::Adapter::Util::logging_methods() ) {
+    no strict 'refs';
+    *{$method} = sub {
+        my ( $self, $msg ) = @_;
+        push(
+            @msgs,
+            {
+                message  => $msg,
+                level    => $method,
+                category => $self->{category}
+            }
+        );
+    };
 }
 
 # Testing methods below
@@ -70,7 +66,27 @@ sub contains_ok {
     else {
         $tb->ok( 0, $test_name );
         $tb->diag( "could not find message matching $regex; log contains: "
-              . _dump_one_line( $self->msgs ) );
+              . $self->dump_one_line( $self->msgs ) );
+    }
+}
+
+sub category_contains_ok {
+    my ( $self, $category, $regex, $test_name ) = @_;
+
+    $test_name ||= "log for $category contains '$regex'";
+    my $found =
+      _first_index(
+        sub { $_->{category} eq $category && $_->{message} =~ /$regex/ },
+        @{ $self->msgs } );
+    if ( $found != -1 ) {
+        splice( @{ $self->msgs }, $found, 1 );
+        $tb->ok( 1, $test_name );
+    }
+    else {
+        $tb->ok( 0, $test_name );
+        $tb->diag(
+            "could not find $category message matching $regex; log contains: "
+              . $self->dump_one_line( $self->msgs ) );
     }
 }
 
@@ -89,6 +105,24 @@ sub does_not_contain_ok {
     }
 }
 
+sub category_does_not_contain_ok {
+    my ( $self, $category, $regex, $test_name ) = @_;
+
+    $test_name ||= "log for $category contains '$regex'";
+    my $found =
+      _first_index(
+        sub { $_->{category} eq $category && $_->{message} =~ /$regex/ },
+        @{ $self->msgs } );
+    if ( $found != -1 ) {
+        $tb->ok( 0, $test_name );
+        $tb->diag( "found $category message matching $regex: "
+              . $self->msgs->[$found] );
+    }
+    else {
+        $tb->ok( 1, $test_name );
+    }
+}
+
 sub empty_ok {
     my ( $self, $test_name ) = @_;
 
@@ -98,8 +132,8 @@ sub empty_ok {
     }
     else {
         $tb->ok( 0, $test_name );
-        $tb->diag(
-            "log is not empty; contains " . _dump_one_line( $self->msgs ) );
+        $tb->diag( "log is not empty; contains "
+              . $self->dump_one_line( $self->msgs ) );
         $self->clear();
     }
 }
@@ -115,26 +149,11 @@ sub contains_only_ok {
     }
     else {
         $tb->ok( 0, $test_name );
-        $tb->diag(
-            "log contains $count messages: " . _dump_one_line( $self->msgs ) );
+        $tb->diag( "log contains $count messages: "
+              . $self->dump_one_line( $self->msgs ) );
     }
 }
 
-sub _dump_one_line {
-    my ($value) = @_;
-
-    return Data::Dumper->new( [$value] )->Indent(0)->Sortkeys(1)->Quotekeys(0)
-      ->Terse(1)->Dump();
-}
-
-sub _make_method {
-    my ( $method, $code, $pkg ) = @_;
-
-    $pkg ||= caller();
-    no strict 'refs';
-    *{ $pkg . "::$method" } = $code;
-}
-
 sub _first_index {
     my $f = shift;
     for my $i ( 0 .. $#_ ) {
